@@ -8,7 +8,7 @@ package Tickit::Term;
 use strict;
 use warnings;
 
-our $VERSION = '0.60';
+our $VERSION = '0.61';
 
 use Carp;
 
@@ -212,9 +212,6 @@ Inserts this event handler first in the chain, before any existing ones.
 
 =back
 
-(This behaviour used to only be available via a separately named method called
-C<bind_event_with_flags> but that name will shortly be removed.)
-
 =head2 unbind_event_id
 
    $term->unbind_event_id( $id )
@@ -234,9 +231,7 @@ sub bind_event
 
 sub bind_event_with_flags
 {
-   my $self = shift;
-   warnings::warnif( deprecated => "\term->bind_event_with_flags is deprecated; use ->bind_event" );
-   $self->_bind_event( @_ );
+   croak "\term->bind_event_with_flags is deprecated; use ->bind_event";
 }
 
 sub bind_event_default
@@ -455,7 +450,7 @@ Sets the terminal window icon text, title, or both.
 
    $term->input_push_bytes( $bytes )
 
-Feeds more bytes of input. May result in C<on_key> or C<on_mouse> events.
+Feeds more bytes of input. May result in C<key> or C<mouse> events.
 
 =cut
 
@@ -464,7 +459,7 @@ Feeds more bytes of input. May result in C<on_key> or C<on_mouse> events.
    $term->input_readable
 
 Informs the term that the input handle may be readable. Attempts to read more
-bytes of input. May result in C<on_key> or C<on_mouse> events.
+bytes of input. May result in C<key> or C<mouse> events.
 
 =cut
 
@@ -473,7 +468,7 @@ bytes of input. May result in C<on_key> or C<on_mouse> events.
    $term->input_wait( $timeout )
 
 Block until some input is available, and process it. Returns after one round
-of input has been processed. May result in C<on_key> or C<on_mouse> events. If
+of input has been processed. May result in C<key> or C<mouse> events. If
 C<$timeout> is defined, it will wait a period of time no longer than this time
 before returning, even if no input events were received.
 
@@ -485,7 +480,7 @@ before returning, even if no input events were received.
 
 Returns a number in seconds to represent when the next timeout should occur on
 the terminal, or C<undef> if nothing is waiting. May invoke expired timeouts,
-and cause a C<on_key> event to occur.
+and cause a C<key> event to occur.
 
 =cut
 
@@ -534,6 +529,28 @@ sub emit_mouse
    ) );
 }
 
+=head1 EVENTS
+
+The following event types are emitted and may be observed by L</bind_event>.
+
+=head2 resize
+
+Emitted when the terminal itself has been resized.
+
+=head2 key
+
+Emitted when a key on the keyboard is pressed.
+
+=head2 mouse
+
+Emitted when a mouse button is pressed or released, the cursor moved while a
+button is held (a dragging event), or the wheel is scrolled.
+
+Behaviour of events involving more than one mouse button is not well-specified
+by terminals.
+
+=cut
+
 =head1 TODO
 
 =over 4
