@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use 5.010; # //
 
-our $VERSION = '0.60';
+our $VERSION = '0.61';
 
 use Carp;
 
@@ -39,8 +39,10 @@ C<Tickit::Window> - a window for drawing operations
 
  my $rootwin = $tickit->rootwin;
 
- $rootwin->set_on_expose( sub {
-    my ( $win, $rb, $rect ) = @_;
+ $rootwin->bind_event( expose => sub {
+    my ( $win, undef, $info ) = @_;
+
+    my $rb = $info->rb;
 
     $rb->clear;
 
@@ -49,7 +51,7 @@ C<Tickit::Window> - a window for drawing operations
        "Hello, world"
     );
  });
- $rootwin->set_on_geom_changed( sub { shift->expose } );
+ $rootwin->bind_event( geomchange => sub { shift->expose } );
  $rootwin->set_pen( Tickit::Pen->new( fg => "white" ) );
 
  $rootwin->expose;
@@ -209,8 +211,8 @@ sub _on_later
 
    $win->close
 
-Removes the window from its parent and clears any event handlers set using any
-of the C<set_on_*> methods. Also recursively closes any child windows.
+Removes the window from its parent and clears any event handlers set using
+L<bind_event>. Also recursively closes any child windows.
 
 Currently this is an optional method, as child windows are stored as weakrefs,
 so should be destroyed when the last reference to them is dropped. Widgets
@@ -223,8 +225,6 @@ sub _close
 {
    my $self = shift;
 
-   $self->unbind_event_id( $_ ) for values %{ $self->{event_ids} };
-
    defined and $_->_close for @{ $self->{child_windows} };
    undef $self->{child_windows};
 }
@@ -435,8 +435,6 @@ The type of C<$info> will depend on the kind of event that was received, as
 indicated by C<$ev>. The information structure types are documented in
 L<Tickit::Event>.
 
-This method is preferred over the legacy C<set_on_EVENT> handler methods.
-
 =head2 bind_event (with flags)
 
    $id = $win->bind_event( $ev, $flags, $code, $data )
@@ -798,92 +796,14 @@ sub change_geometry
    }
 }
 
-=head2 set_on_geom_changed
-
-   $win->set_on_geom_changed( $on_geom_changed )
-
-Set the callback to invoke whenever the window is resized or repositioned;
-i.e. whenever its geometry changes.
-
- $on_geom_changed->( $win )
-
-This method is now deprecated in favour of L</bind_event>.
-
-=cut
-
 sub set_on_geom_changed
 {
-   my $self = shift;
-   my ( $code ) = @_;
-
-   warnings::warnif( deprecated => "\$win->set_on_geom_changed is deprecated; use ->bind_event" );
-
-   $self->unbind_event_id( delete $self->{event_ids}{geomchange} ) if $self->{event_ids}{geomchange};
-   $self->{event_ids}{geomchange} = $self->bind_event( geomchange => sub {
-      my ( $self ) = @_;
-      $code->( $self );
-   } );
+   croak "\$win->set_on_geom_changed is deprecated; use ->bind_event";
 }
 
-=head2 set_on_key
-
-   $win->set_on_key( $on_key )
-
-   $win->set_on_key( with_ev => $on_key )
-
-Set the callback to invoke whenever a key is pressed while this window, or one
-of its child windows, has the input focus. The callback will be passed the
-window and an event structure.
-
- $handled = $on_key->( $win, $event )
-
-C<$event> is an event structure object supporting C<type>, C<str> and C<mod>.
-C<type> will be C<"text"> for normal unmodified Unicode, or C<"key"> for
-special keys or modified Unicode. C<str> will be the UTF-8 string for C<text>
-events, or the textual description of the key as rendered by L<Term::TermKey>
-for C<key> events. C<mod> will be a bitmask of the modifier state.
-
-The invoked code should return a true value if it considers the keypress dealt
-with, or false to pass it up to its parent window.
-
-Before passing it to its parent, a window will also try any other non-focused
-sibling windows of the currently-focused window in order of creation (though
-note this order is not necessarily the order the child widgets that own those
-windows were created or added to their container).
-
-If no window actually handles the keypress, then every window will eventually
-be consulted about it, preferring windows closer to the focused one.
-
-This broadcast-like behaviour allows widgets to handle keypresses that should
-make sense even though their window does not actually have the keyboard focus.
-This feature should be used sparingly, to only capture one or two keypresses
-that really make sense; for example to capture the C<PageUp> and C<PageDown>
-keys in a scrolling list, or a numbered function key that performs some
-special action.
-
-The C<with_ev>-prefixed form is accepted for backward compatibility.
-
-This method is now deprecated in favour of L</bind_event>.
-
-=cut
-
 sub set_on_key
 {
-   my $self = shift;
-
-   if( @_ > 1 and $_[0] eq "with_ev" ) {
-      shift;
-   }
-
-   warnings::warnif( deprecated => "\$win->set_on_key is deprecated; use ->bind_event" );
-
-   my ( $code ) = @_;
-
-   $self->unbind_event_id( delete $self->{event_ids}{key} ) if $self->{event_ids}{key};
-   $self->{event_ids}{key} = $self->bind_event( key => sub {
-      my ( $self, undef, $info ) = @_;
-      $code->( $self, $info );
-   } ) if $code;
+   croak "\$win->set_on_key is deprecated; use ->bind_event";
 }
 
 sub _handle_key
@@ -919,99 +839,9 @@ sub _handle_key
    return 0;
 }
 
-=head2 set_on_mouse
-
-   $win->set_on_mouse( $on_mouse )
-
-   $win->set_on_mouse( with_ev => $on_mouse )
-
-Set the callback to invoke whenever a mouse event is received within the
-window's rectangle. The callback will be passed the window and an event
-structure.
-
- $handled = $on_mouse->( $win, $event )
-
-C<$event> is an event structure object supporting C<type>, C<button>, C<line>,
-C<col> and C<mod>.
-
-Behaviour of events involving more than one mouse button is
-not well-specified by terminals.
-
-The following event names may be observed:
-
-=over 8
-
-=item press
-
-A mouse button has been pressed down on this cell
-
-=item drag_start
-
-The mouse was moved while a button was held, and was initially in the given
-cell
-
-=item drag
-
-The mouse was moved while a button was held, and is now in the given cell
-
-=item drag_outside
-
-The mouse was moved outside of the window that handled the C<drag_start>
-event, and is still being dragged.
-
-=item drag_drop
-
-A mouse button was released after having been moved, while in the given cell
-
-=item drag_stop
-
-The drag operation has finished. This event is always given directly to the
-window that handled the C<drag_start> event, rather than the window on which
-the mouse release event happened.
-
-=item release
-
-A mouse button was released after being pressed
-
-=item wheel
-
-The mouse wheel was moved. C<button> will indicate the wheel direction as a
-string C<up> or C<down>.
-
-=back
-
-The invoked code should return a true value if it considers the mouse event
-dealt with, or false to pass it up to its parent window.
-
-Once a dragging operation has begun via C<drag_start>, the window that handled
-the event will always receive C<drag>, C<drag_outside>, and an eventual
-C<drag_stop> event even if the mouse moves outside that window. No other
-window will receive a C<drag_outside> or C<drag_stop> event than the one that
-started the operation.
-
-The C<with_ev>-prefixed form is accepted for backward compatibility.
-
-This method is now deprecated in favour of L</bind_event>.
-
-=cut
-
 sub set_on_mouse
 {
-   my $self = shift;
-
-   if( @_ > 1 and $_[0] eq "with_ev" ) {
-      shift;
-   }
-
-   warnings::warnif( deprecated => "\$win->set_on_mouse is deprecated; use ->bind_event" );
-
-   my ( $code ) = @_;
-
-   $self->unbind_event_id( delete $self->{event_ids}{mouse} ) if $self->{event_ids}{mouse};
-   $self->{event_ids}{mouse} = $self->bind_event( mouse => sub {
-      my ( $self, undef, $info ) = @_;
-      $code->( $self, $info );
-   } ) if $code;
+   croak "\$win->set_on_mouse is deprecated; use ->bind_event";
 }
 
 sub _handle_mouse
@@ -1050,49 +880,9 @@ sub _handle_mouse
    return 0;
 }
 
-=head2 set_on_expose
-
-   $win->set_on_expose( $on_expose )
-
-   $win->set_on_expose( with_rb => $on_expose )
-
-Set the callback to invoke whenever a region of the window is exposed by the
-C<expose> event. When invoked, it is passed the window itself, a
-L<Tickit::RenderBuffer> and a L<Tickit::Rect> representing the portion of the
-window that was exposed.
-
- $on_expose->( $win, $rb, $rect )
-
-The buffer's origin will be that of the window, and its clipping region will
-already be set to the expose rect. This will automatically be flushed to the
-window when the callback returns.
-
-The C<with_rb>-prefixed form is accepted for backward compatibility.
-
-If any child windows overlap the region, these will be exposed first, before
-the containing window.
-
-This method is now deprecated in favour of L</bind_event>.
-
-=cut
-
 sub set_on_expose
 {
-   my $self = shift;
-
-   if( @_ > 1 and $_[0] eq "with_rb" ) {
-      shift;
-   }
-
-   warnings::warnif( deprecated => "\$win->set_on_expose is deprecated; use ->bind_event" );
-
-   my ( $code ) = @_;
-
-   $self->unbind_event_id( delete $self->{event_ids}{expose} ) if $self->{event_ids}{expose};
-   $self->{event_ids}{expose} = $self->bind_event( expose => sub {
-      my ( $self, undef, $info ) = @_;
-      $code->( $self, $info->rb, $info->rect );
-   } ) if $code;
+   croak "\$win->set_on_expose is deprecated; use ->bind_event";
 }
 
 our $INDENT = "";
@@ -1177,46 +967,9 @@ sub expose
    $self->_needs_later;
 }
 
-=head2 set_on_focus
-
-   $win->set_on_focus( $on_refocus )
-
-Set the callback to invoke whenever the window gains or loses focus.
-
- $on_refocus->( $win, $has_focus )
-
-Will be passed a boolean value, true if the focus was just gained, false if
-the focus was just lost.
-
-If the C<focus_child_notify> behavior is enabled, this callback is also
-invoked for changes of focus on descendent windows. In this case, it is
-passed an additional argument, being the immediate child window in which the
-focus chain has now changed (which may or may not be the focused window
-directly; it could itself be another ancestor).
-
- $on_refocus->( $win, $has_focus, $child )
-
-When a window gains focus, any of its ancestors that have
-C<focus_child_notify> enabled will be informed first, from the outermost
-inwards, before the window itself. When one loses focus, it is notified
-first, and then its parents from the innermost outwards.
-
-This method is now deprecated in favour of L</bind_event>.
-
-=cut
-
 sub set_on_focus
 {
-   my $self = shift;
-   my ( $code ) = @_;
-
-   warnings::warnif( deprecated => "\$win->set_on_focus is deprecated; use ->bind_event" );
-
-   $self->unbind_event_id( delete $self->{event_ids}{focus} ) if $self->{event_ids}{focus};
-   $self->{event_ids}{focus} = $self->bind_event( focus => sub {
-      my ( $self, undef, $info ) = @_;
-      $code->( $self, $info->type, $info->win );
-   } ) if $code;
+   croak "\$win->set_on_focus is deprecated; use ->bind_event";
 }
 
 =head2 set_focus_child_notify
@@ -1937,50 +1690,24 @@ sub restore
    $term->flush;
 }
 
-=head2 clearline
-
-   $win->clearline( $line )
-
-Erase the entire content of one line of the window
-
-=cut
-
 sub clearline
 {
-   my $self = shift;
-   my ( $line ) = @_;
-
-   return unless $self->{visible};
-
-   $self->goto( $line, 0 );
-   $self->erasech( $self->cols );
+   croak "\$win->clearline has been removed";
 }
 
-=head2 clear
-
-   $win->clear
-
-Erase the entire content of the window and reset it to the current background
-colour.
-
-=cut
-
 sub clear
 {
    my $self = shift;
 
-   return unless $self->{visible};
+   croak "\$win->clear on non-root window has been removed" if $self->parent;
 
-   if( $self->parent ) {
-      $self->clearline( $_ ) for 0 .. $self->lines - 1;
-   }
-   else {
-      my $term = $self->term;
-      $term->setpen( $self->get_effective_pen );
-      $term->clear;
+   carp "\$win->clear on root window is deprecated and will be removed";
 
-      $self->_needs_flush;
-   }
+   my $term = $self->term;
+   $term->setpen( $self->get_effective_pen );
+   $term->clear;
+
+   $self->_needs_flush;
 }
 
 sub _needs_flush
@@ -2014,6 +1741,124 @@ use overload
    bool => sub { 1 },
    fallback => 1;
 
+=head1 EVENTS
+
+The following event types are emitted and may be observed by L</bind_event>.
+
+=head2 key
+
+Emitted when a key on the keyboard is pressed while this window or one of its
+child windows has the input focus, or is set to steal input anyway.
+
+The event handler should return a true value if it considers the keypress
+dealt with, or false to pass it up to its parent window.
+
+Before passing it to its parent, a window will also try any other non-focused
+sibling windows of the currently-focused window in order of creation (though
+note this order is not necessarily the order the child widgets that own those
+windows were created or added to their container).
+
+If no window actually handles the keypress, then every window will eventually
+be consulted about it, preferring windows closer to the focused one.
+
+This broadcast-like behaviour allows widgets to handle keypresses that should
+make sense even though their window does not actually have the keyboard focus.
+This feature should be used sparingly, to only capture one or two keypresses
+that really make sense; for example to capture the C<PageUp> and C<PageDown>
+keys in a scrolling list, or a numbered function key that performs some
+special action.
+
+=head2 mouse
+
+Emitted when a mouse button is pressed or released, the cursor moved while a
+button is held (a dragging event), or the wheel is scrolled.
+
+The following event names may be observed:
+
+=over 8
+
+=item press
+
+A mouse button has been pressed down on this cell
+
+=item drag_start
+
+The mouse was moved while a button was held, and was initially in the given
+cell
+
+=item drag
+
+The mouse was moved while a button was held, and is now in the given cell
+
+=item drag_outside
+
+The mouse was moved outside of the window that handled the C<drag_start>
+event, and is still being dragged.
+
+=item drag_drop
+
+A mouse button was released after having been moved, while in the given cell
+
+=item drag_stop
+
+The drag operation has finished. This event is always given directly to the
+window that handled the C<drag_start> event, rather than the window on which
+the mouse release event happened.
+
+=item release
+
+A mouse button was released after being pressed
+
+=item wheel
+
+The mouse wheel was moved. C<button> will indicate the wheel direction as a
+string C<up> or C<down>.
+
+=back
+
+The invoked code should return a true value if it considers the mouse event
+dealt with, or false to pass it up to its parent window.
+
+Once a dragging operation has begun via C<drag_start>, the window that handled
+the event will always receive C<drag>, C<drag_outside>, and an eventual
+C<drag_stop> event even if the mouse moves outside that window. No other
+window will receive a C<drag_outside> or C<drag_stop> event than the one that
+started the operation.
+
+=head2 geomchange
+
+Emitted when the window is resized or repositioned; i.e. whenever its geometry
+changes.
+
+=head2 expose
+
+Emitted when a region of the window is exposed by the L<expose> method, or
+implicitly because it or another window has changed size, been shown or
+hidden, or the stacking order has been changed.
+
+When invoked, render buffer passed in the event will have its origin set to
+that of the window, and its clipping will be set to the damage rectangle.
+
+If any child windows overlap the region, these will be exposed first, before
+the containing window.
+
+=head2 focus
+
+Emitted when the window gains or loses input focus.
+
+If the C<focus_child_notify> behavior is enabled, this callback is also
+invoked for changes of focus on descendent windows. In this case, it is
+passed an additional argument, being the immediate child window in which the
+focus chain has now changed (which may or may not be the focused window
+directly; it could itself be another ancestor).
+
+When a window gains focus, any of its ancestors that have
+C<focus_child_notify> enabled will be informed first, from the outermost
+inwards, before the window itself. When one loses focus, it is notified
+first, and then its parents from the innermost outwards.
+
+=cut
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
