@@ -99,7 +99,7 @@ TickitPen *tickit_pen_clone(const TickitPen *orig)
   return pen;
 }
 
-static void tickit_pen_destroy(TickitPen *pen)
+static void destroy(TickitPen *pen)
 {
   tickit_hooklist_unbind_and_destroy(pen->hooks, pen);
   free(pen);
@@ -115,7 +115,7 @@ void tickit_pen_unref(TickitPen *pen)
 {
   pen->refcount--;
   if(!pen->refcount)
-    tickit_pen_destroy(pen);
+    destroy(pen);
 }
 
 bool tickit_pen_has_attr(const TickitPen *pen, TickitPenAttr attr)
